@@ -3,10 +3,12 @@ import logging
 
 from functools import partial
 
-from typing import Callable, Any, Generator
+from typing import Callable, Any
 from aio_pika.channel import Channel
 from aio_pika.queue import Queue
-from aio_pika.message import IncomingMessage, Message, DeliveryMode, ReturnedMessage
+from aio_pika.message import (
+    IncomingMessage, Message, DeliveryMode, ReturnedMessage
+)
 
 from .base import Proxy, Base
 
@@ -14,6 +16,20 @@ from .base import Proxy, Base
 log = logging.getLogger(__name__)
 
 
+class MessageProcessingError(Exception):
+    pass
+
+
+class NackMessage(MessageProcessingError):
+    def __init__(self, requeue=False):
+        self.requeue = requeue
+
+
+class RejectMessage(MessageProcessingError):
+    def __init__(self, requeue=False):
+        self.requeue = requeue
+
+
 class Worker:
     __slots__ = 'queue', 'consumer_tag', 'loop',
 
@@ -27,9 +43,8 @@ class Worker:
 
         :return: :class:`asyncio.Task`
         """
-        @asyncio.coroutine
-        def closer():
-            yield from self.queue.cancel(self.consumer_tag)
+        async def closer():
+            await self.queue.cancel(self.consumer_tag)
 
         return self.loop.create_task(closer())
 
@@ -63,8 +78,13 @@ class Master(Base):
         self.channel = channel          # type: Channel
         self.loop = self.channel.loop   # type: asyncio.AbstractEventLoop
         self.proxy = Proxy(self.create_task)
+
         self.channel.add_on_return_callback(self.on_message_returned)
 
+    @property
+    def exchange(self):
+        return self.channel.default_exchange
+
     def on_message_returned(self, message: ReturnedMessage):
         log.warning(
             "Message returned. Probably destination queue does not exists: %r",
@@ -92,29 +112,36 @@ class Master(Base):
         return super().deserialize(data)
 
     @classmethod
-    @asyncio.coroutine
-    def execute(cls, func, kwargs):
+    async def execute(cls, func, kwargs):
         kwargs = kwargs or {}
-        result = yield from func(**kwargs)
+        result = await func(**kwargs)
         return result
 
-    @asyncio.coroutine
-    def on_message(self, func, message: IncomingMessage):
+    async def on_message(self, func, message: IncomingMessage):
         with message.process(requeue=True, ignore_processed=True):
             data = self.deserialize(message.body)
-            yield from self.execute(func, data)
 
-    @asyncio.coroutine
-    def create_worker(self, channel_name: str,
-                      func: Callable, **kwargs) -> Generator[Any, None, Worker]:
+            try:
+                await self.execute(func, data)
+            except RejectMessage as e:
+                message.reject(requeue=e.requeue)
+            except NackMessage as e:
+                message.nack(requeue=e.requeue)
+
+    async def create_queue(self, channel_name, **kwargs) -> Queue:
+        return await self.channel.declare_queue(channel_name, **kwargs)
+
+    async def create_worker(self, channel_name: str,
+                            func: Callable, **kwargs) -> Worker:
         """ Creates a new :class:`Worker` instance. """
-        queue = yield from self.channel.declare_queue(channel_name, **kwargs)
+
+        queue = await self.create_queue(channel_name, **kwargs)
 
         if hasattr(func, "_is_coroutine"):
             fn = func
         else:
             fn = asyncio.coroutine(func)
-        consumer_tag = yield from queue.consume(
+        consumer_tag = await queue.consume(
             partial(
                 self.on_message,
                 fn
@@ -123,8 +150,9 @@ class Master(Base):
 
         return Worker(queue, consumer_tag, self.loop)
 
-    @asyncio.coroutine
-    def create_task(self, channel_name: str, kwargs=None, **message_kwargs):
+    async def create_task(self, channel_name: str,
+                          kwargs=None, **message_kwargs):
+
         """ Creates a new task for the worker """
         message = Message(
             body=self.serialize(kwargs or {}),
@@ -133,6 +161,6 @@ class Master(Base):
             **message_kwargs
         )
 
-        yield from self.channel.default_exchange.publish(
+        await self.exchange.publish(
             message, channel_name, mandatory=True
         )
